@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2002, 2003, 2005, 2007, 2008 John W. Eaton
+Copyright (C) 2002-2011 John W. Eaton
 
 This file is part of Octave.
 
@@ -53,7 +53,7 @@ octave_throw_interrupt_exception (void)
 {
   if (octave_interrupt_hook)
     octave_interrupt_hook ();
-    
+
   throw octave_interrupt_exception ();
 }
 
@@ -63,7 +63,7 @@ octave_throw_execution_exception (void)
   // FIXME -- would a hook function be useful here?
 
   octave_exception_state = octave_exec_exception;
-    
+
   throw octave_execution_exception ();
 }
 
@@ -72,7 +72,7 @@ octave_throw_bad_alloc (void)
 {
   if (octave_bad_alloc_hook)
     octave_bad_alloc_hook ();
-    
+
   octave_exception_state = octave_alloc_exception;
 
   throw std::bad_alloc ();
@@ -82,27 +82,24 @@ void
 octave_rethrow_exception (void)
 {
   if (octave_interrupt_state)
-    octave_throw_interrupt_exception ();
+    {
+      octave_interrupt_state = -1;
+      octave_throw_interrupt_exception ();
+    }
   else
     {
       switch (octave_exception_state)
-	{
-	case octave_exec_exception:
-	  octave_throw_execution_exception ();
-	  break;
+        {
+        case octave_exec_exception:
+          octave_throw_execution_exception ();
+          break;
 
-	case octave_alloc_exception:
-	  octave_throw_bad_alloc ();
-	  break;
+        case octave_alloc_exception:
+          octave_throw_bad_alloc ();
+          break;
 
-	default:
-	  break;
-	}
+        default:
+          break;
+        }
     }
 }
-
-/*
-;;; Local Variables: ***
-;;; mode: C++ ***
-;;; End: ***
-*/
