@@ -1,7 +1,6 @@
 /*
 
-Copyright (C) 1994, 1995, 1996, 1997, 2000, 2002, 2004, 2005, 2007
-              John W. Eaton
+Copyright (C) 1994-2011 John W. Eaton
 
 This file is part of Octave.
 
@@ -30,10 +29,12 @@ along with Octave; see the file COPYING.  If not, see
 #include "CRowVector.h"
 #include "dColVector.h"
 #include "CColVector.h"
+#include "DET.h"
 
 #include "mx-defs.h"
 
 class
+OCTAVE_API
 ComplexDiagMatrix : public MDiagArray2<Complex>
 {
 public:
@@ -45,17 +46,14 @@ public:
   ComplexDiagMatrix (octave_idx_type r, octave_idx_type c, const Complex& val)
     : MDiagArray2<Complex> (r, c, val) { }
 
-  explicit ComplexDiagMatrix (const RowVector& a)
-    : MDiagArray2<Complex> (ComplexRowVector (a)) { }
-
-  explicit ComplexDiagMatrix (const ComplexRowVector& a)
+  explicit ComplexDiagMatrix (const Array<Complex>& a)
     : MDiagArray2<Complex> (a) { }
 
-  explicit ComplexDiagMatrix (const ColumnVector& a)
-    : MDiagArray2<Complex> (ComplexColumnVector (a)) { }
+  explicit ComplexDiagMatrix (const Array<double>& a)
+    : MDiagArray2<Complex> (Array<Complex> (a)) { }
 
-  explicit ComplexDiagMatrix (const ComplexColumnVector& a)
-    : MDiagArray2<Complex> (a) { }
+  ComplexDiagMatrix (const Array<Complex>& a, octave_idx_type r, octave_idx_type c)
+    : MDiagArray2<Complex> (a, r, c) { }
 
   explicit ComplexDiagMatrix (const DiagMatrix& a);
 
@@ -65,6 +63,10 @@ public:
   ComplexDiagMatrix (const ComplexDiagMatrix& a)
     : MDiagArray2<Complex> (a) { }
 
+  template <class U>
+  ComplexDiagMatrix (const DiagArray2<U>& a)
+    : MDiagArray2<Complex> (a) { }
+
   ComplexDiagMatrix& operator = (const ComplexDiagMatrix& a)
     {
       MDiagArray2<Complex>::operator = (a);
@@ -87,10 +89,11 @@ public:
   ComplexDiagMatrix& fill (const RowVector& a, octave_idx_type beg);
   ComplexDiagMatrix& fill (const ComplexRowVector& a, octave_idx_type beg);
 
-  ComplexDiagMatrix hermitian (void) const;  // complex conjugate transpose
-  ComplexDiagMatrix transpose (void) const;
+  ComplexDiagMatrix hermitian (void) const { return MDiagArray2<Complex>::hermitian (std::conj); }
+  ComplexDiagMatrix transpose (void) const { return MDiagArray2<Complex>::transpose(); }
+  DiagMatrix abs (void) const;
 
-  friend ComplexDiagMatrix conj (const ComplexDiagMatrix& a);
+  friend OCTAVE_API ComplexDiagMatrix conj (const ComplexDiagMatrix& a);
 
   // resize is the destructive analog for this one
 
@@ -104,8 +107,11 @@ public:
   ComplexColumnVector column (octave_idx_type i) const;
   ComplexColumnVector column (char *s) const;
 
-  ComplexDiagMatrix inverse (int& info) const;
+  ComplexDiagMatrix inverse (octave_idx_type& info) const;
   ComplexDiagMatrix inverse (void) const;
+  ComplexDiagMatrix pseudo_inverse (void) const;
+
+  bool all_elements_are_real (void) const;
 
   // diagonal matrix by diagonal matrix -> diagonal matrix operations
 
@@ -114,36 +120,31 @@ public:
 
   // other operations
 
-  ComplexColumnVector diag (void) const;
-  ComplexColumnVector diag (octave_idx_type k) const;
+  ComplexColumnVector diag (octave_idx_type k = 0) const
+    { return MDiagArray2<Complex>::diag (k); }
+
+  ComplexDET determinant (void) const;
+  double rcond (void) const;
 
   // i/o
 
   friend std::ostream& operator << (std::ostream& os, const ComplexDiagMatrix& a);
 
-private:
-
-  ComplexDiagMatrix (Complex *d, octave_idx_type nr, octave_idx_type nc)
-    : MDiagArray2<Complex> (d, nr, nc) { }
 };
 
+OCTAVE_API ComplexDiagMatrix conj (const ComplexDiagMatrix& a);
+
 // diagonal matrix by diagonal matrix -> diagonal matrix operations
 
-ComplexDiagMatrix
+OCTAVE_API ComplexDiagMatrix
 operator * (const ComplexDiagMatrix& a, const ComplexDiagMatrix& b);
 
-ComplexDiagMatrix
+OCTAVE_API ComplexDiagMatrix
 operator * (const ComplexDiagMatrix& a, const DiagMatrix& b);
 
-ComplexDiagMatrix
+OCTAVE_API ComplexDiagMatrix
 operator * (const DiagMatrix& a, const ComplexDiagMatrix& b);
 
 MDIAGARRAY2_FORWARD_DEFS (MDiagArray2, ComplexDiagMatrix, Complex)
 
 #endif
-
-/*
-;;; Local Variables: ***
-;;; mode: C++ ***
-;;; End: ***
-*/
