@@ -1,7 +1,6 @@
 /*
 
-Copyright (C) 1994, 1995, 1996, 1997, 2000, 2002, 2004, 2005, 2006,
-              2007 John W. Eaton
+Copyright (C) 1994-2011 John W. Eaton
 
 This file is part of Octave.
 
@@ -24,9 +23,11 @@ along with Octave; see the file COPYING.  If not, see
 #if !defined (octave_QRP_h)
 #define octave_QRP_h 1
 
-#include <iostream>
+#include <iosfwd>
 
 #include "dbleQR.h"
+#include "PermMatrix.h"
+#include "dColVector.h"
 
 class
 OCTAVE_API
@@ -36,38 +37,34 @@ public:
 
   QRP (void) : QR (), p () { }
 
-  QRP (const Matrix&, QR::type = QR::std);
+  QRP (const Matrix&, qr_type_t = qr_type_std);
 
   QRP (const QRP& a) : QR (a), p (a.p) { }
 
   QRP& operator = (const QRP& a)
     {
       if (this != &a)
-	{
-	  QR::operator = (a);
-	  p = a.p;
-	}
+        {
+          QR::operator = (a);
+          p = a.p;
+        }
 
       return *this;
     }
 
   ~QRP (void) { }
 
-  void init (const Matrix&, QR::type = QR::std);
+  void init (const Matrix&, qr_type_t = qr_type_std);
 
-  Matrix P (void) const { return p; }
+  PermMatrix P (void) const { return p; }
+
+  RowVector Pvec (void) const;
 
   friend std::ostream&  operator << (std::ostream&, const QRP&);
 
 protected:
 
-  Matrix p;
+  PermMatrix p;
 };
 
 #endif
-
-/*
-;;; Local Variables: ***
-;;; mode: C++ ***
-;;; End: ***
-*/
