@@ -1,7 +1,7 @@
 // N-D Array  manipulations.
 /*
 
-Copyright (C) 2004, 2005, 2006, 2007 John W. Eaton
+Copyright (C) 2004-2011 John W. Eaton
 
 This file is part of Octave.
 
@@ -26,9 +26,11 @@ along with Octave; see the file COPYING.  If not, see
 #endif
 
 #include "int32NDArray.h"
-
+#include "mx-op-defs.h"
 #include "intNDArray.cc"
 
+#include "bsxfun-defs.cc"
+
 template class OCTAVE_API intNDArray<octave_int32>;
 
 template OCTAVE_API
@@ -39,17 +41,16 @@ template OCTAVE_API
 std::istream&
 operator >> (std::istream& is, intNDArray<octave_int32>& a);
 
-NDS_CMP_OPS (int32NDArray, , octave_int32, )
-NDS_BOOL_OPS (int32NDArray, octave_int32, octave_int32 (0))
+NDS_CMP_OPS (int32NDArray, octave_int32)
+NDS_BOOL_OPS (int32NDArray, octave_int32)
 
-SND_CMP_OPS (octave_int32, , int32NDArray, )
-SND_BOOL_OPS (octave_int32, int32NDArray, octave_int32 (0))
+SND_CMP_OPS (octave_int32, int32NDArray)
+SND_BOOL_OPS (octave_int32, int32NDArray)
 
-NDND_CMP_OPS (int32NDArray, , int32NDArray, )
-NDND_BOOL_OPS (int32NDArray, int32NDArray, octave_int32 (0))
+NDND_CMP_OPS (int32NDArray, int32NDArray)
+NDND_BOOL_OPS (int32NDArray, int32NDArray)
 
-/*
-;;; Local Variables: ***
-;;; mode: C++ ***
-;;; End: ***
-*/
+MINMAX_FCNS (int32NDArray, octave_int32)
+
+BSXFUN_STDOP_DEFS_MXLOOP (int32NDArray)
+BSXFUN_STDREL_DEFS_MXLOOP (int32NDArray)
