@@ -1,7 +1,7 @@
 // N-D Array  manipulations.
 /*
 
-Copyright (C) 2004, 2005, 2006, 2007, 2009 John W. Eaton
+Copyright (C) 2004-2011 John W. Eaton
 
 This file is part of Octave.
 
@@ -29,6 +29,8 @@ along with Octave; see the file COPYING.  If not, see
 #include "mx-op-defs.h"
 #include "intNDArray.cc"
 
+#include "bsxfun-defs.cc"
+
 template class OCTAVE_API intNDArray<octave_uint64>;
 
 template OCTAVE_API
@@ -39,19 +41,16 @@ template OCTAVE_API
 std::istream&
 operator >> (std::istream& is, intNDArray<octave_uint64>& a);
 
-NDS_CMP_OPS (uint64NDArray, , octave_uint64, )
-NDS_BOOL_OPS (uint64NDArray, octave_uint64, octave_uint64 (0))
+NDS_CMP_OPS (uint64NDArray, octave_uint64)
+NDS_BOOL_OPS (uint64NDArray, octave_uint64)
 
-SND_CMP_OPS (octave_uint64, , uint64NDArray, )
-SND_BOOL_OPS (octave_uint64, uint64NDArray, octave_uint64 (0))
+SND_CMP_OPS (octave_uint64, uint64NDArray)
+SND_BOOL_OPS (octave_uint64, uint64NDArray)
 
-NDND_CMP_OPS (uint64NDArray, , uint64NDArray, )
-NDND_BOOL_OPS (uint64NDArray, uint64NDArray, octave_uint64 (0))
+NDND_CMP_OPS (uint64NDArray, uint64NDArray)
+NDND_BOOL_OPS (uint64NDArray, uint64NDArray)
 
-MINMAX_FCNS (uint64)
+MINMAX_FCNS (uint64NDArray, octave_uint64)
 
-/*
-;;; Local Variables: ***
-;;; mode: C++ ***
-;;; End: ***
-*/
+BSXFUN_STDOP_DEFS_MXLOOP (uint64NDArray)
+BSXFUN_STDREL_DEFS_MXLOOP (uint64NDArray)
