@@ -21,28 +21,29 @@
 ## @deftypefnx {Function File} {@var{entries} =} gzip (@var{files}, @var{outdir})
 ## Compress the list of files and/or directories specified in @var{files}.
 ## Each file is compressed separately and a new file with a '.gz' extension
-## is created.  The original files are not touched.  Existing compressed
+## is created.  The original files are not modified.  Existing compressed
 ## files are silently overwritten.  If @var{outdir} is defined the compressed
-## versions of the files are placed in this directory.
+## files are placed in this directory.
 ## @seealso{gunzip, bzip2, zip, tar}
 ## @end deftypefn
 
 function entries = gzip (varargin)
-  if (nargin == 1 || nargin == 2) && (nargout <= 1)
-    if nargout == 0
-      __xzip__ ("gzip", "gz", "gzip -r %s", varargin{:});
-    else
-      entries = __xzip__ ("gzip", "gz", "gzip -r %s", varargin{:});
-    endif
-  else
+  if (nargin != 1 && nargin != 2) || (nargout > 1)
     print_usage ();
   endif
+     
+  if (nargout == 0)
+    __xzip__ ("gzip", "gz", "gzip -r %s", varargin{:});
+  else
+    entries = __xzip__ ("gzip", "gz", "gzip -r %s", varargin{:});
+  endif
+
 endfunction
 
 %!error <Invalid call to gzip.  Correct usage is> gzip("1", "2", "3");
 %!error <Invalid call to gzip.  Correct usage is> gzip();
 %!error <output directory does not exist> gzip("1", tmpnam);
-%!error <expecting FILES to be a character array> gzip(1);
+%!error <FILES must be a character array or cellstr> gzip(1);
 %!xtest
 %!  # test gzip together with gunzip
 %!  unwind_protect
