@@ -1,5 +1,4 @@
-## Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2002,
-##               2003, 2004, 2005, 2006, 2007 Shai Ayal
+## Copyright (C) 1993-2011 Shai Ayal
 ##
 ## This file is part of Octave.
 ##
@@ -18,11 +17,12 @@
 ## <http://www.gnu.org/licenses/>.
 
 ## -*- texinfo -*-
-## @deftypefn {Function File} {@var{c} =} contour (@var{z})
-## @deftypefnx {Function File} {@var{c} =} contour (@var{z}, @var{vn})
-## @deftypefnx {Function File} {@var{c} =} contour (@var{x}, @var{y}, @var{z})
-## @deftypefnx {Function File} {@var{c} =} contour (@var{x}, @var{y}, @var{z}, @var{vn})
-## @deftypefnx {Function File} {@var{c} =} contour (@var{h}, @dots{})
+## @deftypefn  {Function File} {} contour (@var{z})
+## @deftypefnx {Function File} {} contour (@var{z}, @var{vn})
+## @deftypefnx {Function File} {} contour (@var{x}, @var{y}, @var{z})
+## @deftypefnx {Function File} {} contour (@var{x}, @var{y}, @var{z}, @var{vn})
+## @deftypefnx {Function File} {} contour (@dots{}, @var{style})
+## @deftypefnx {Function File} {} contour (@var{h}, @dots{})
 ## @deftypefnx {Function File} {[@var{c}, @var{h}] =} contour (@dots{})
 ## Plot level curves (contour lines) of the matrix @var{z}, using the
 ## contour matrix @var{c} computed by @code{contourc} from the same
@@ -38,13 +38,17 @@
 ## @end group
 ## @end example
 ##
-## The optional input and output argument @var{h} allows an axis handle to 
+## The style to use for the plot can be defined with a line style @var{style}
+## in a similar manner to the line styles used with the @code{plot} command.
+## Any markers defined by @var{style} are ignored.
+##
+## The optional input and output argument @var{h} allows an axis handle to
 ## be passed to @code{contour} and the handles to the contour objects to be
 ## returned.
 ## @seealso{contourc, patch, plot}
 ## @end deftypefn
 
-## Author: shaia
+## Author: Shai Ayal <shaiay@users.sourceforge.net>
 
 function [c, h] = contour (varargin)
 
@@ -54,7 +58,7 @@ function [c, h] = contour (varargin)
   unwind_protect
     axes (xh);
     newplot ();
-    [ctmp, htmp] = __contour__ (xh, NaN, varargin{:});
+    [ctmp, htmp] = __contour__ (xh, "none", varargin{:});
   unwind_protect_cleanup
     axes (oldh);
   end_unwind_protect
@@ -69,3 +73,9 @@ endfunction
 %!demo
 %! [x, y, z] = peaks ();
 %! contour (x, y, z);
+
+%!demo
+%! [theta, r] = meshgrid (linspace (0, 2*pi, 64), linspace(0,1,64));
+%! [X, Y] = pol2cart (theta, r);
+%! Z = sin(2*theta).*(1-r);
+%! contour(X, Y, abs(Z), 10)
