@@ -20,18 +20,17 @@
 ## -*- texinfo -*-
 ## @deftypefn  {Function File} {} pascal (@var{n})
 ## @deftypefnx {Function File} {} pascal (@var{n}, @var{t})
-## Return the Pascal matrix of order @var{n} if @code{@var{t} = 0}.
-## @var{t} defaults to 0.  Return lower triangular Cholesky@tie{}factor of
-## the Pascal matrix if @code{@var{t} = 1}.  This matrix is its own
-## inverse, that is @code{pascal (@var{n}, 1) ^ 2 == eye (@var{n})}.
-## If @code{@var{t} = -1}, return its absolute value.  This is the
-## standard Pascal triangle as a lower-triangular matrix.
-## If @code{@var{t} = 2}, return a transposed and permuted version of
-## @code{pascal (@var{n}, 1)}, which is the cube-root of the identity
-## matrix.  That is @code{pascal (@var{n}, 2) ^ 3 == eye (@var{n})}.
+## Return the Pascal matrix of order @var{n} if @code{@var{t} = 0}.  @var{t}
+## defaults to 0.  Return the pseudo-lower triangular Cholesky@tie{}factor of
+## the Pascal matrix if @code{@var{t} = 1} (The sign of some columns may be
+## negative).  This matrix is its own inverse, that is @code{pascal (@var{n},
+## 1) ^ 2 == eye (@var{n})}.  If @code{@var{t} = -1}, return the true
+## Cholesky@tie{}factor with strictly positive values on the diagonal.  If
+## @code{@var{t} = 2}, return a transposed and permuted version of @code{pascal
+## (@var{n}, 1)}, which is the cube root of the identity matrix.  That is,
+## @code{pascal (@var{n}, 2) ^ 3 == eye (@var{n})}.
 ##
-## @seealso{hankel, vander, sylvester_matrix, hilb, invhilb, toeplitz
-##          hadamard, wilkinson, compan, rosser}
+## @seealso{chol}
 ## @end deftypefn
 
 ## Author: Peter Ekberg
@@ -52,7 +51,7 @@ function retval = pascal (n, t)
   endif
 
   if (t < -1 || t > 2)
-    error ("pascal: expecting t to be -1, 0, 1, or 2, found %d", t);
+    error ("pascal: expecting T to be -1, 0, 1, or 2, found %d", t);
   endif
 
   retval = zeros (n);
