@@ -64,6 +64,10 @@ filter (MArray<T>& b, MArray<T>& a, MArray<T>& x, MArray<T>& si,
 
   octave_idx_type ab_len = a_len > b_len ? a_len : b_len;
 
+  // FIXME: The two lines below should be unecessary because
+  //        this template is called with a and b as column vectors
+  //        already.  However the a.resize line is currently (2011/04/26)
+  //        necessary to stop bug #33164.
   b.resize (dim_vector (ab_len, 1), 0.0);
   if (a_len > 1)
     a.resize (dim_vector (ab_len, 1), 0.0);
@@ -79,7 +83,7 @@ filter (MArray<T>& b, MArray<T>& a, MArray<T>& x, MArray<T>& si,
   dim_vector x_dims = x.dims ();
   if (dim < 0 || dim > x_dims.length ())
     {
-      error ("filter: filtering over invalid dimension");
+      error ("filter: DIM must be a valid dimension");
       return y;
     }
 
@@ -262,7 +266,7 @@ filter (MArray<T>& b, MArray<T>& a, MArray<T>& x, int dim = -1)
   else
     if (dim < 0 || dim > x_dims.length ())
       {
-        error ("filter: filtering over invalid dimension");
+        error ("filter: DIM must be a valid dimension");
         return MArray<T> ();
       }
 
@@ -399,7 +403,7 @@ $$\n\
       dim = args(4).nint_value() - 1;
       if (dim < 0 || dim >= x_dims.length ())
         {
-          error ("filter: filtering over invalid dimension");
+          error ("filter: DIM must be a valid dimension");
           return retval;
         }
     }
@@ -464,7 +468,7 @@ $$\n\
                   si = args(3).float_complex_array_value ();
 
                   if (si_is_vector)
-                    si = si.reshape (dim_vector (si.numel (), 1));
+                    si = si.reshape (dim_vector (1, si.numel ()));
                 }
 
               if (! error_state)
@@ -521,7 +525,7 @@ $$\n\
                   si = args(3).complex_array_value ();
 
                   if (si_is_vector)
-                    si = si.reshape (dim_vector (si.numel (), 1));
+                    si = si.reshape (dim_vector (1, si.numel ()));
                 }
 
               if (! error_state)
@@ -581,7 +585,7 @@ $$\n\
                   si = args(3).float_array_value ();
 
                   if (si_is_vector)
-                    si = si.reshape (dim_vector (si.numel (), 1));
+                    si = si.reshape (dim_vector (1, si.numel ()));
                 }
 
               if (! error_state)
@@ -638,7 +642,7 @@ $$\n\
                   si = args(3).array_value ();
 
                   if (si_is_vector)
-                    si = si.reshape (dim_vector (si.numel (), 1));
+                    si = si.reshape (dim_vector (1, si.numel ()));
                 }
 
               if (! error_state)
@@ -694,58 +698,59 @@ filter (MArray<FloatComplex>&, MArray<FloatComplex>&, MArray<FloatComplex>&, int
 %!test
 %!  a = [1 1];
 %!  b = [1 1];
-%!  x = zeros(1,10); x(1) = 1;
-%!  assert(all(filter(b,   [1], x  ) == [1 1 0 0 0 0 0 0 0 0]  ))
-%!  assert(all(filter(b,   [1], x.') == [1 1 0 0 0 0 0 0 0 0].'))
-%!  assert(all(filter(b.', [1], x  ) == [1 1 0 0 0 0 0 0 0 0]  ))
-%!  assert(all(filter(b.', [1], x.') == [1 1 0 0 0 0 0 0 0 0].'))
-%!  assert(all(filter([1], a,   x  ) == [+1 -1 +1 -1 +1 -1 +1 -1 +1 -1]  ))
-%!  assert(all(filter([1], a,   x.') == [+1 -1 +1 -1 +1 -1 +1 -1 +1 -1].'))
-%!  assert(all(filter([1], a.', x  ) == [+1 -1 +1 -1 +1 -1 +1 -1 +1 -1]  ))
-%!  assert(all(filter([1], a.', x.') == [+1 -1 +1 -1 +1 -1 +1 -1 +1 -1].'))
-%!  assert(all(filter(b,   a,   x  ) == [1 0 0 0 0 0 0 0 0 0]  ))
-%!  assert(all(filter(b.', a,   x  ) == [1 0 0 0 0 0 0 0 0 0]  ))
-%!  assert(all(filter(b,   a.', x  ) == [1 0 0 0 0 0 0 0 0 0]  ))
-%!  assert(all(filter(b.', a,   x  ) == [1 0 0 0 0 0 0 0 0 0]  ))
-%!  assert(all(filter(b,   a,   x.') == [1 0 0 0 0 0 0 0 0 0].'))
-%!  assert(all(filter(b.', a,   x.') == [1 0 0 0 0 0 0 0 0 0].'))
-%!  assert(all(filter(b,   a.', x.') == [1 0 0 0 0 0 0 0 0 0].'))
-%!  assert(all(filter(b.', a,   x.') == [1 0 0 0 0 0 0 0 0 0].'))
+%!  x = zeros (1,10); x(1) = 1;
+%!  assert(filter(b,   [1], x  ), [1 1 0 0 0 0 0 0 0 0]);
+%!  assert(filter(b,   [1], x.'), [1 1 0 0 0 0 0 0 0 0].');
+%!  assert(filter(b.', [1], x  ), [1 1 0 0 0 0 0 0 0 0]  );
+%!  assert(filter(b.', [1], x.'), [1 1 0 0 0 0 0 0 0 0].');
+%!  assert(filter([1], a,   x  ), [+1 -1 +1 -1 +1 -1 +1 -1 +1 -1]  );
+%!  assert(filter([1], a,   x.'), [+1 -1 +1 -1 +1 -1 +1 -1 +1 -1].');
+%!  assert(filter([1], a.', x  ), [+1 -1 +1 -1 +1 -1 +1 -1 +1 -1]  );
+%!  assert(filter([1], a.', x.'), [+1 -1 +1 -1 +1 -1 +1 -1 +1 -1].');
+%!  assert(filter(b,   a,   x  ), [1 0 0 0 0 0 0 0 0 0]  );
+%!  assert(filter(b.', a,   x  ), [1 0 0 0 0 0 0 0 0 0]  );
+%!  assert(filter(b,   a.', x  ), [1 0 0 0 0 0 0 0 0 0]  );
+%!  assert(filter(b.', a,   x  ), [1 0 0 0 0 0 0 0 0 0]  );
+%!  assert(filter(b,   a,   x.'), [1 0 0 0 0 0 0 0 0 0].');
+%!  assert(filter(b.', a,   x.'), [1 0 0 0 0 0 0 0 0 0].');
+%!  assert(filter(b,   a.', x.'), [1 0 0 0 0 0 0 0 0 0].');
+%!  assert(filter(b.', a,   x.'), [1 0 0 0 0 0 0 0 0 0].');
 %!
 %!test
 %!  r = sqrt(1/2)*(1+i);
 %!  a = a*r;
 %!  b = b*r;
-%!  assert(all(filter(b, [1], x   ) == r*[1 1 0 0 0 0 0 0 0 0]   ))
-%!  assert(all(filter(b, [1], r*x ) == r*r*[1 1 0 0 0 0 0 0 0 0] ))
-%!  assert(all(filter(b, [1], x.' ) == r*[1 1 0 0 0 0 0 0 0 0].' ))
-%!  assert(all(filter(b, a,   x   ) ==   [1 0 0 0 0 0 0 0 0 0]   ))
-%!  assert(all(filter(b, a,   r*x ) == r*[1 0 0 0 0 0 0 0 0 0]   ))
+%!  assert(filter(b, [1], x   ), r*[1 1 0 0 0 0 0 0 0 0]   );
+%!  assert(filter(b, [1], r*x ), r*r*[1 1 0 0 0 0 0 0 0 0] );
+%!  assert(filter(b, [1], x.' ), r*[1 1 0 0 0 0 0 0 0 0].' );
+%!  assert(filter(b, a,   x   ),   [1 0 0 0 0 0 0 0 0 0]   );
+%!  assert(filter(b, a,   r*x ), r*[1 0 0 0 0 0 0 0 0 0]   );
 %!
 %!shared a, b, x, y, so
 %!test
-%!  a = [1,1]; b=[1,1];
-%!  x = zeros(1,10); x(1) = 1;
-%!  [y, so] = filter(b, [1], x, [-1]);
-%!  assert(all(y == [0 1 0 0 0 0 0 0 0 0]))
-%!  assert(so,0)
+%!  a = [1,1]; b = [1,1];
+%!  x = zeros (1,10); x(1) = 1;
+%!  [y, so] = filter (b, [1], x, [-1]);
+%!  assert(y, [0 1 0 0 0 0 0 0 0 0]);
+%!  assert(so,0);
 %!
 %!test
-%!  x  = zeros(10,3); x(1,1)=-1; x(1,2)=1;
-%!  y0 = zeros(10,3); y0(1:2,1)=-1; y0(1:2,2)=1;
-%!  y = filter(b,[1],x);
-%!  assert(all(all(y==y0)))
+%!  x  = zeros (10,3); x(1,1)=-1; x(1,2)=1;
+%!  y0 = zeros (10,3); y0(1:2,1)=-1; y0(1:2,2)=1;
+%!  y = filter (b, [1], x);
+%!  assert(y,y0);
 %!
 %!test
 %!  a = [1,1]; b=[1,1];
-%!  x = zeros(4,4,2); x(1,1:4,1) = +1; x(1,1:4,2) = -1;
-%!  y0 = zeros(4,4,2); y0(1:2,1:4,1) = +1; y0(1:2,1:4,2) = -1;
-%!  y = filter(b, [1], x);
-%!  assert(all(all(all(y==y0))))
+%!  x = zeros (4,4,2); x(1,1:4,1) = +1; x(1,1:4,2) = -1;
+%!  y0 = zeros (4,4,2); y0(1:2,1:4,1) = +1; y0(1:2,1:4,2) = -1;
+%!  y = filter (b, [1], x);
+%!  assert(y, y0);
 %!
-%!assert(filter(1,ones(10,1)/10,[]), [])
-%!assert(filter(1,ones(10,1)/10,zeros(0,10)), zeros(0,10))
-
+%!assert(filter (1, ones(10,1)/10, []), []);
+%!assert(filter (1, ones(10,1)/10, zeros(0,10)), zeros(0,10));
+%!assert(filter([1, 3], [1], [1 2; 3 4; 5 6], [4, 5]), [5 7; 6 10; 14 18]);
+%!assert(filter (1, ones(10,1)/10, single (1:5)), repmat (single (10), 1, 5));
 %%  Should put some tests of the "DIM" parameter in here.
 
 */
