@@ -1,7 +1,6 @@
 /*
 
-Copyright (C) 1996, 1997, 1998, 2000, 2002, 2003, 2004, 2005, 2007, 2008
-              John W. Eaton
+Copyright (C) 1996-2011 John W. Eaton
 
 This file is part of Octave.
 
@@ -45,13 +44,13 @@ DEFUNOP (transpose, char_matrix_str)
       return octave_value ();
     }
   else
-    return octave_value (v.char_matrix_value().transpose (), true,
-			 a.is_sq_string () ? '\'' : '"');
+    return octave_value (v.char_matrix_value().transpose (),
+                         a.is_sq_string () ? '\'' : '"');
 }
 
 // string by string ops.
 
-#define DEFCHARNDBINOP_FN(name, op, t1, t2, e1, e2, f)	\
+#define DEFCHARNDBINOP_FN(name, op, t1, t2, e1, e2, f)  \
   BINOPDECL (name, a1, a2) \
   { \
     dim_vector a1_dims = a1.dims (); \
@@ -64,17 +63,17 @@ DEFUNOP (transpose, char_matrix_str)
  \
     if (a1_is_scalar) \
       { \
-	if (a2_is_scalar) \
-	  return octave_value ((v1.e1 ## _value ())(0) op (v2.e2 ## _value ())(0)); \
-	else \
-	  return octave_value (f ((v1.e1 ## _value ())(0), v2.e2 ## _value ())); \
+        if (a2_is_scalar) \
+          return octave_value ((v1.e1 ## _value ())(0) op (v2.e2 ## _value ())(0)); \
+        else \
+          return octave_value (f ((v1.e1 ## _value ())(0), v2.e2 ## _value ())); \
       } \
     else \
       { \
-	if (a2_is_scalar) \
-	  return octave_value (f (v1.e1 ## _value (), (v2.e2 ## _value ())(0))); \
-	else \
-	  return octave_value (f (v1.e1 ## _value (), v2.e2 ## _value ())); \
+        if (a2_is_scalar) \
+          return octave_value (f (v1.e1 ## _value (), (v2.e2 ## _value ())(0))); \
+        else \
+          return octave_value (f (v1.e1 ## _value (), v2.e2 ## _value ())); \
       } \
   }
 
@@ -89,7 +88,7 @@ DEFASSIGNOP (assign, char_matrix_str, char_matrix_str)
 {
   CAST_BINOP_ARGS (octave_char_matrix_str&, const octave_char_matrix_str&);
 
-  v1.assign (idx, v2.char_matrix_value ());
+  v1.assign (idx, v2.char_array_value ());
   return octave_value ();
 }
 
@@ -110,27 +109,27 @@ install_str_str_ops (void)
   INSTALL_BINOP (op_lt, octave_char_matrix_str, octave_char_matrix_sq_str, lt);
   INSTALL_BINOP (op_lt, octave_char_matrix_sq_str, octave_char_matrix_str, lt);
   INSTALL_BINOP (op_lt, octave_char_matrix_sq_str, octave_char_matrix_sq_str, lt);
-  
+
   INSTALL_BINOP (op_le, octave_char_matrix_str, octave_char_matrix_str, le);
   INSTALL_BINOP (op_le, octave_char_matrix_str, octave_char_matrix_sq_str, le);
   INSTALL_BINOP (op_le, octave_char_matrix_sq_str, octave_char_matrix_str, le);
   INSTALL_BINOP (op_le, octave_char_matrix_sq_str, octave_char_matrix_sq_str, le);
-  
+
   INSTALL_BINOP (op_eq, octave_char_matrix_str, octave_char_matrix_str, eq);
   INSTALL_BINOP (op_eq, octave_char_matrix_str, octave_char_matrix_sq_str, eq);
   INSTALL_BINOP (op_eq, octave_char_matrix_sq_str, octave_char_matrix_str, eq);
   INSTALL_BINOP (op_eq, octave_char_matrix_sq_str, octave_char_matrix_sq_str, eq);
-  
+
   INSTALL_BINOP (op_ge, octave_char_matrix_str, octave_char_matrix_str, ge);
   INSTALL_BINOP (op_ge, octave_char_matrix_str, octave_char_matrix_sq_str, ge);
   INSTALL_BINOP (op_ge, octave_char_matrix_sq_str, octave_char_matrix_str, ge);
   INSTALL_BINOP (op_ge, octave_char_matrix_sq_str, octave_char_matrix_sq_str, ge);
-  
+
   INSTALL_BINOP (op_gt, octave_char_matrix_str, octave_char_matrix_str, gt);
   INSTALL_BINOP (op_gt, octave_char_matrix_str, octave_char_matrix_sq_str, gt);
   INSTALL_BINOP (op_gt, octave_char_matrix_sq_str, octave_char_matrix_str, gt);
   INSTALL_BINOP (op_gt, octave_char_matrix_sq_str, octave_char_matrix_sq_str, gt);
-  
+
   INSTALL_BINOP (op_ne, octave_char_matrix_str, octave_char_matrix_str, ne);
   INSTALL_BINOP (op_ne, octave_char_matrix_str, octave_char_matrix_sq_str, ne);
   INSTALL_BINOP (op_ne, octave_char_matrix_sq_str, octave_char_matrix_str, ne);
@@ -154,9 +153,3 @@ install_str_str_ops (void)
   INSTALL_ASSIGNOP (op_asn_eq, octave_char_matrix_sq_str, octave_null_sq_str, null_assign);
 
 }
-
-/*
-;;; Local Variables: ***
-;;; mode: C++ ***
-;;; End: ***
-*/
