@@ -1,7 +1,6 @@
 /*
 
-Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
-              2002, 2004, 2005, 2006, 2007, 2008, 2009 John W. Eaton
+Copyright (C) 1993-2011 John W. Eaton
 
 This file is part of Octave.
 
@@ -65,7 +64,7 @@ public:
 
 private:
 
-  octave_shlib_list (void) { }
+  octave_shlib_list (void) : lib_list () { }
 
   ~octave_shlib_list (void) { }
 
@@ -101,18 +100,20 @@ octave_shlib_list::do_append (const octave_shlib& shl)
 
 void
 octave_shlib_list::do_remove (octave_shlib& shl,
-			      octave_shlib::close_hook cl_hook)
+                              octave_shlib::close_hook cl_hook)
 {
   for (iterator p = lib_list.begin (); p != lib_list.end (); p++)
     {
       if (*p == shl)
-	{
-	  shl.close (cl_hook);
+        {
+          // Erase first to avoid potentially invalidating the pointer by the
+          // following hooks.
+          lib_list.erase (p);
 
-	  lib_list.erase (p);
+          shl.close (cl_hook);
 
-	  break;
-	}
+          break;
+        }
     }
 }
 
@@ -124,10 +125,10 @@ octave_shlib_list::do_find_file (const std::string& file_name) const
   for (const_iterator p = lib_list.begin (); p != lib_list.end (); p++)
     {
       if (p->file_name () == file_name)
-	{
-	  retval = *p;
-	  break;
-	}
+        {
+          retval = *p;
+          break;
+        }
     }
 
   return retval;
@@ -168,7 +169,7 @@ octave_shlib_list::append (const octave_shlib& shl)
 
 void
 octave_shlib_list::remove (octave_shlib& shl,
-			   octave_shlib::close_hook cl_hook)
+                           octave_shlib::close_hook cl_hook)
 {
   if (instance_ok ())
     instance->do_remove (shl, cl_hook);
@@ -202,7 +203,7 @@ public:
 
 private:
 
-  octave_mex_file_list (void) { }
+  octave_mex_file_list (void) : file_list () { }
 
   ~octave_mex_file_list (void) { }
 
@@ -234,18 +235,20 @@ octave_mex_file_list::do_append (const octave_shlib& shl)
 
 void
 octave_mex_file_list::do_remove (octave_shlib& shl,
-				 octave_shlib::close_hook cl_hook)
+                                 octave_shlib::close_hook cl_hook)
 {
   for (iterator p = file_list.begin (); p != file_list.end (); p++)
     {
       if (*p == shl)
-	{
-	  shl.close (cl_hook);
+        {
+          // Erase first to avoid potentially invalidating the pointer by the
+          // following hooks.
+          file_list.erase (p);
 
-	  file_list.erase (p);
+          shl.close (cl_hook);
 
-	  break;
-	}
+          break;
+        }
     }
 }
 
@@ -276,7 +279,7 @@ octave_mex_file_list::append (const octave_shlib& shl)
 
 void
 octave_mex_file_list::remove (octave_shlib& shl,
-			      octave_shlib::close_hook cl_hook)
+                              octave_shlib::close_hook cl_hook)
 {
   if (instance_ok ())
     instance->do_remove (shl, cl_hook);
@@ -304,35 +307,39 @@ octave_dynamic_loader::instance_ok (void)
   return retval;
 }
 
-static
-void do_clear_function (const std::string& fcn_name)
+static void
+do_clear_function (const std::string& fcn_name)
 {
   warning_with_id ("Octave:reload-forces-clear", "  %s", fcn_name.c_str ());
 
-  symbol_table::clear_user_function (fcn_name);
+  symbol_table::clear_dld_function (fcn_name);
 }
 
 static void
 clear (octave_shlib& oct_file)
 {
   if (oct_file.number_of_functions_loaded () > 1)
-    warning_with_id ("Octave:reload-forces-clear",
-		     "reloading %s clears the following functions:",
-		     oct_file.file_name().c_str ());
+    {
+      warning_with_id ("Octave:reload-forces-clear",
+                       "reloading %s clears the following functions:",
+                       oct_file.file_name().c_str ());
 
-  octave_shlib_list::remove (oct_file, do_clear_function);
+      octave_shlib_list::remove (oct_file, do_clear_function);
+    }
+  else
+    octave_shlib_list::remove (oct_file, symbol_table::clear_dld_function);
 }
 
 octave_function *
 octave_dynamic_loader::do_load_oct (const std::string& fcn_name,
-				    const std::string& file_name,
-				    bool relative)
+                                    const std::string& file_name,
+                                    bool relative)
 {
   octave_function *retval = 0;
 
-  unwind_protect::begin_frame ("octave_dynamic_loader::do_load");
+  unwind_protect frame;
 
-  unwind_protect_bool (octave_dynamic_loader::doing_load);
+  frame.protect_var (octave_dynamic_loader::doing_load);
 
   doing_load = true;
 
@@ -346,60 +353,53 @@ octave_dynamic_loader::do_load_oct (const std::string& fcn_name,
       oct_file.open (file_name);
 
       if (! error_state && oct_file)
-	{
-	  octave_shlib_list::append (oct_file);
-
-	  if (relative)
-	    oct_file.mark_relative ();
-	}
+        octave_shlib_list::append (oct_file);
     }
 
   if (! error_state)
     {
       if (oct_file)
-	{
-	  void *function = oct_file.search (fcn_name, name_mangler);
+        {
+          void *function = oct_file.search (fcn_name, name_mangler);
 
-	  if (! function)
-	    {
-	      // FIXME -- can we determine this C mangling scheme
-	      // automatically at run time or configure time?
+          if (! function)
+            {
+              // FIXME -- can we determine this C mangling scheme
+              // automatically at run time or configure time?
 
-	      function = oct_file.search (fcn_name, name_uscore_mangler);
-	    }
+              function = oct_file.search (fcn_name, name_uscore_mangler);
+            }
 
-	  if (function)
-	    {
-	      octave_dld_fcn_getter f
-		= FCN_PTR_CAST (octave_dld_fcn_getter, function);
+          if (function)
+            {
+              octave_dld_fcn_getter f
+                = FCN_PTR_CAST (octave_dld_fcn_getter, function);
 
-	      retval = f (oct_file, relative);
+              retval = f (oct_file, relative);
 
-	      if (! retval)
-		::error ("failed to install .oct file function `%s'",
-			 fcn_name.c_str ());
-	    }
-	}
+              if (! retval)
+                ::error ("failed to install .oct file function `%s'",
+                         fcn_name.c_str ());
+            }
+        }
       else
-	::error ("%s is not a valid shared library",
-		 file_name.c_str ());
+        ::error ("%s is not a valid shared library",
+                 file_name.c_str ());
     }
-  
-  unwind_protect::run_frame ("octave_dynamic_loader::do_load");
 
   return retval;
 }
 
 octave_function *
 octave_dynamic_loader::do_load_mex (const std::string& fcn_name,
-				    const std::string& file_name,
-				    bool relative)
+                                    const std::string& file_name,
+                                    bool /*relative*/)
 {
   octave_function *retval = 0;
 
-  unwind_protect::begin_frame ("octave_dynamic_loader::do_load");
+  unwind_protect frame;
 
-  unwind_protect_bool (octave_dynamic_loader::doing_load);
+  frame.protect_var (octave_dynamic_loader::doing_load);
 
   doing_load = true;
 
@@ -413,62 +413,55 @@ octave_dynamic_loader::do_load_mex (const std::string& fcn_name,
       mex_file.open (file_name);
 
       if (! error_state && mex_file)
-	{
-	  octave_shlib_list::append (mex_file);
-
-	  if (relative)
-	    mex_file.mark_relative ();
-	}
+        octave_shlib_list::append (mex_file);
     }
 
   if (! error_state)
     {
       if (mex_file)
-	{
-	  void *function = 0;
+        {
+          void *function = 0;
 
-	  bool have_fmex = false;
+          bool have_fmex = false;
 
-	  octave_mex_file_list::append (mex_file);
+          octave_mex_file_list::append (mex_file);
 
-	  function = mex_file.search (fcn_name, mex_mangler);
+          function = mex_file.search (fcn_name, mex_mangler);
 
-	  if (! function)
-	    {
-	      // FIXME -- can we determine this C mangling scheme
-	      // automatically at run time or configure time?
+          if (! function)
+            {
+              // FIXME -- can we determine this C mangling scheme
+              // automatically at run time or configure time?
 
-	      function = mex_file.search (fcn_name, mex_uscore_mangler);
+              function = mex_file.search (fcn_name, mex_uscore_mangler);
 
-	      if (! function)
-		{
-		  function = mex_file.search (fcn_name, mex_f77_mangler);
+              if (! function)
+                {
+                  function = mex_file.search (fcn_name, mex_f77_mangler);
 
-		  if (function)
-		    have_fmex = true;
-		}
-	    }
+                  if (function)
+                    have_fmex = true;
+                }
+            }
 
-	  if (function)
-	    retval = new octave_mex_function (function, have_fmex,
-					      mex_file, fcn_name);
-	  else
-	    ::error ("failed to install .mex file function `%s'",
-		     fcn_name.c_str ());
-  	}
+          if (function)
+            retval = new octave_mex_function (function, have_fmex,
+                                              mex_file, fcn_name);
+          else
+            ::error ("failed to install .mex file function `%s'",
+                     fcn_name.c_str ());
+        }
       else
-	::error ("%s is not a valid shared library",
-		 file_name.c_str ());
+        ::error ("%s is not a valid shared library",
+                 file_name.c_str ());
     }
 
-  unwind_protect::run_frame ("octave_dynamic_loader::do_load");
-
   return retval;
 }
 
 bool
 octave_dynamic_loader::do_remove_oct (const std::string& fcn_name,
-				      octave_shlib& shl)
+                                      octave_shlib& shl)
 {
   bool retval = false;
 
@@ -480,7 +473,7 @@ octave_dynamic_loader::do_remove_oct (const std::string& fcn_name,
       retval = shl.remove (fcn_name);
 
       if (shl.number_of_functions_loaded () == 0)
-	octave_shlib_list::remove (shl);
+        octave_shlib_list::remove (shl);
     }
 
   return retval;
@@ -488,7 +481,7 @@ octave_dynamic_loader::do_remove_oct (const std::string& fcn_name,
 
 bool
 octave_dynamic_loader::do_remove_mex (const std::string& fcn_name,
-				      octave_shlib& shl)
+                                      octave_shlib& shl)
 {
   bool retval = false;
 
@@ -500,7 +493,7 @@ octave_dynamic_loader::do_remove_mex (const std::string& fcn_name,
       retval = shl.remove (fcn_name);
 
       if (shl.number_of_functions_loaded () == 0)
-	octave_mex_file_list::remove (shl);
+        octave_mex_file_list::remove (shl);
     }
 
   return retval;
@@ -508,8 +501,8 @@ octave_dynamic_loader::do_remove_mex (const std::string& fcn_name,
 
 octave_function *
 octave_dynamic_loader::load_oct (const std::string& fcn_name,
-				  const std::string& file_name,
-				  bool relative)
+                                  const std::string& file_name,
+                                  bool relative)
 {
   return (instance_ok ())
     ? instance->do_load_oct (fcn_name, file_name, relative) : 0;
@@ -517,8 +510,8 @@ octave_dynamic_loader::load_oct (const std::string& fcn_name,
 
 octave_function *
 octave_dynamic_loader::load_mex (const std::string& fcn_name,
-				  const std::string& file_name,
-				  bool relative)
+                                  const std::string& file_name,
+                                  bool relative)
 {
   return (instance_ok ())
     ? instance->do_load_mex (fcn_name, file_name, relative) : 0;
@@ -526,14 +519,14 @@ octave_dynamic_loader::load_mex (const std::string& fcn_name,
 
 bool
 octave_dynamic_loader::remove_oct (const std::string& fcn_name,
-				   octave_shlib& shl)
+                                   octave_shlib& shl)
 {
   return (instance_ok ()) ? instance->do_remove_oct (fcn_name, shl) : false;
 }
 
 bool
 octave_dynamic_loader::remove_mex (const std::string& fcn_name,
-				   octave_shlib& shl)
+                                   octave_shlib& shl)
 {
   return (instance_ok ()) ? instance->do_remove_mex (fcn_name, shl) : false;
 }
@@ -567,9 +560,3 @@ octave_dynamic_loader::mex_f77_mangler (const std::string&)
 {
   return STRINGIFY (F77_FUNC (mexfunction, MEXFUNCTION));
 }
-
-/*
-;;; Local Variables: ***
-;;; mode: C++ ***
-;;; End: ***
-*/
