@@ -1,7 +1,6 @@
 /*
 
-Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
-              2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 John W. Eaton
+Copyright (C) 1993-2011 John W. Eaton
 
 This file is part of Octave.
 
@@ -36,12 +35,8 @@ along with Octave; see the file COPYING.  If not, see
 #include <sstream>
 #include <string>
 
-#ifdef HAVE_UNISTD_H
-#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
-#endif
 #include <unistd.h>
-#endif
 
 #include "cmd-edit.h"
 #include "file-ops.h"
@@ -55,6 +50,7 @@ along with Octave; see the file COPYING.  If not, see
 #include "gripes.h"
 #include "help.h"
 #include "input.h"
+#include "lex.h"
 #include "load-path.h"
 #include "oct-map.h"
 #include "oct-hist.h"
@@ -112,9 +108,15 @@ bool input_from_eval_string_pending = false;
 // the command line.
 bool input_from_command_line_file = false;
 
+// TRUE means that stdin is a terminal, not a pipe or redirected file.
+bool stdin_is_tty = false;
+
 // TRUE means we're parsing a function file.
 bool reading_fcn_file = false;
 
+// TRUE means we're parsing a classdef file.
+bool reading_classdef_file = false;
+
 // Simple name of function file we are reading.
 std::string curr_fcn_file_name;
 
@@ -169,29 +171,29 @@ do_input_echo (const std::string& input_string)
   if (do_echo)
     {
       if (forced_interactive)
-	{
-	  if (promptflag > 0)
-	    octave_stdout << command_editor::decode_prompt_string (VPS1);
-	  else
-	    octave_stdout << command_editor::decode_prompt_string (VPS2);
-	}
+        {
+          if (promptflag > 0)
+            octave_stdout << command_editor::decode_prompt_string (VPS1);
+          else
+            octave_stdout << command_editor::decode_prompt_string (VPS2);
+        }
       else
-	octave_stdout << command_editor::decode_prompt_string (VPS4);
+        octave_stdout << command_editor::decode_prompt_string (VPS4);
 
       if (! input_string.empty ())
-	{
-	  octave_stdout << input_string;
+        {
+          octave_stdout << input_string;
 
-	  if (input_string[input_string.length () - 1] != '\n')
-	    octave_stdout << "\n";
-	}
+          if (input_string[input_string.length () - 1] != '\n')
+            octave_stdout << "\n";
+        }
     }
 }
 
 std::string
 gnu_readline (const std::string& s, bool force_readline)
 {
-  OCTAVE_QUIT;
+  octave_quit ();
 
   std::string retval;
 
@@ -202,22 +204,22 @@ gnu_readline (const std::string& s, bool force_readline)
       retval = command_editor::readline (s, eof);
 
       if (! eof && retval.empty ())
-	retval = "\n";
+        retval = "\n";
     }
   else
     {
       if (! s.empty () && (interactive || forced_interactive))
-	{
-	  FILE *stream = command_editor::get_output_stream ();
+        {
+          FILE *stream = command_editor::get_output_stream ();
 
-	  fputs (s.c_str (), stream);
-	  fflush (stream);
-	}
+          gnulib::fputs (s.c_str (), stream);
+          fflush (stream);
+        }
 
       FILE *curr_stream = command_editor::get_input_stream ();
 
-      if (reading_fcn_file || reading_script_file)
-	curr_stream = ff_instream;
+      if (reading_fcn_file || reading_script_file || reading_classdef_file)
+        curr_stream = ff_instream;
 
       retval = octave_fgets (curr_stream);
     }
@@ -242,7 +244,7 @@ interactive_input (const std::string& s, bool force_readline = false)
       Vdrawnow_requested = false;
 
       if (error_state)
-	return "\n";
+        return "\n";
     }
 
   return gnu_readline (s, force_readline);
@@ -251,7 +253,7 @@ interactive_input (const std::string& s, bool force_readline = false)
 static std::string
 octave_gets (void)
 {
-  OCTAVE_QUIT;
+  octave_quit ();
 
   std::string retval;
 
@@ -259,9 +261,11 @@ octave_gets (void)
 
   if ((interactive || forced_interactive)
       && (! (reading_fcn_file
-	     || reading_script_file
-	     || input_from_startup_file
-	     || input_from_command_line_file)))
+             || reading_classdef_file
+             || reading_script_file
+             || get_input_from_eval_string
+             || input_from_startup_file
+             || input_from_command_line_file)))
     {
       std::string ps = (promptflag > 0) ? VPS1 : VPS2;
 
@@ -278,20 +282,20 @@ octave_gets (void)
       // There is no need to update the load_path cache if there is no
       // user input.
       if (! retval.empty ()
-	  && retval.find_first_not_of (" \t\n\r") != std::string::npos)
-	{
-	  load_path::update ();
+          && retval.find_first_not_of (" \t\n\r") != std::string::npos)
+        {
+          load_path::update ();
 
-	  if (Vdebugging)
-	    last_debugging_command = retval;
-	  else
-	    last_debugging_command = std::string ();
-	}
+          if (Vdebugging)
+            last_debugging_command = retval;
+          else
+            last_debugging_command = std::string ();
+        }
       else if (Vdebugging)
-	{
-	  retval = last_debugging_command;
-	  history_skip_auto_repeated_debugging_command = true;
-	}
+        {
+          retval = last_debugging_command;
+          history_skip_auto_repeated_debugging_command = true;
+        }
     }
   else
     retval = gnu_readline ("");
@@ -301,22 +305,22 @@ octave_gets (void)
   if (! current_input_line.empty ())
     {
       if (! (input_from_startup_file || input_from_command_line_file
-	     || history_skip_auto_repeated_debugging_command))
-	command_history::add (current_input_line);
+             || history_skip_auto_repeated_debugging_command))
+        command_history::add (current_input_line);
 
-      if (! (reading_fcn_file || reading_script_file))
-	{
-	  octave_diary << current_input_line;
+      if (! (reading_fcn_file || reading_script_file || reading_classdef_file))
+        {
+          octave_diary << current_input_line;
 
-	  if (current_input_line[current_input_line.length () - 1] != '\n')
-	    octave_diary << "\n";
-	}
+          if (current_input_line[current_input_line.length () - 1] != '\n')
+            octave_diary << "\n";
+        }
 
       do_input_echo (current_input_line);
     }
-  else if (! (reading_fcn_file || reading_script_file))
+  else if (! (reading_fcn_file || reading_script_file || reading_classdef_file))
     octave_diary << "\n";
-  
+
   return retval;
 }
 
@@ -325,23 +329,23 @@ octave_gets (void)
 static std::string
 get_user_input (void)
 {
-  OCTAVE_QUIT;
+  octave_quit ();
 
   std::string retval;
 
   if (get_input_from_eval_string)
     {
       if (input_from_eval_string_pending)
-	{
-	  input_from_eval_string_pending = false;
+        {
+          input_from_eval_string_pending = false;
 
-	  retval = current_eval_string;
+          retval = current_eval_string;
 
-	  size_t len = retval.length ();
+          size_t len = retval.length ();
 
-	  if (len > 0 && retval[len-1] != '\n')
-	    retval.append ("\n");
-	}
+          if (len > 0 && retval[len-1] != '\n')
+            retval.append ("\n");
+        }
     }
   else
     retval = octave_gets ();
@@ -385,22 +389,22 @@ octave_read (char *buf, unsigned max_size)
 
       // Make sure input ends with a new line character.
       if (chars_left == 0 && buf[len-1] != '\n')
-	{
-	  if (len < max_size) 
-	    {
-	      // There is enough room to plug the newline character in
-	      // the buffer.
-	      buf[len++] = '\n';
-	    }
-	  else
-	    {
-	      // There isn't enough room to plug the newline character
-	      // in the buffer so make sure it is returned on the next
-	      // octave_read call.
-	      pos = eol;
-	      chars_left = 1;
-	    }
-	}
+        {
+          if (len < max_size)
+            {
+              // There is enough room to plug the newline character in
+              // the buffer.
+              buf[len++] = '\n';
+            }
+          else
+            {
+              // There isn't enough room to plug the newline character
+              // in the buffer so make sure it is returned on the next
+              // octave_read call.
+              pos = eol;
+              chars_left = 1;
+            }
+        }
 
       status = len;
 
@@ -409,7 +413,7 @@ octave_read (char *buf, unsigned max_size)
     {
       status = 0;
     }
-  else    
+  else
     status = -1;
 
   return status;
@@ -429,7 +433,7 @@ get_input_from_file (const std::string& name, int warn)
   if (! instream && warn)
     warning ("%s: no such file or directory", name.c_str ());
 
-  if (reading_fcn_file || reading_script_file)
+  if (reading_fcn_file || reading_script_file || reading_classdef_file)
     ff_instream = instream;
   else
     command_editor::set_input_stream (instream);
@@ -452,7 +456,7 @@ get_input_from_stdin (void)
 
 static string_vector
 generate_possible_completions (const std::string& text, std::string& prefix,
-			       std::string& hint)
+                               std::string& hint)
 {
   string_vector names;
 
@@ -470,6 +474,30 @@ generate_possible_completions (const std::string& text, std::string& prefix,
   return names;
 }
 
+static bool
+is_completing_dirfns (void)
+{
+  static std::string dirfns_commands[] = {"cd", "ls"};
+  static const size_t dirfns_commands_length = 2;
+
+  bool retval = false;
+
+  std::string line = command_editor::get_line_buffer ();
+
+  for (size_t i = 0; i < dirfns_commands_length; i++)
+    {
+      int index = line.find (dirfns_commands[i] + " ");
+
+      if (index == 0)
+        {
+          retval = true;
+          break;
+        }
+    }
+
+  return retval;
+}
+
 static std::string
 generate_completion (const std::string& text, int state)
 {
@@ -496,7 +524,13 @@ generate_completion (const std::string& text, int state)
 
       hint = text;
 
-      name_list = generate_possible_completions (text, prefix, hint);
+      // No reason to display symbols while completing a
+      // file/directory operation.
+
+      if (is_completing_dirfns ())
+        name_list = string_vector ();
+      else
+        name_list = generate_possible_completions (text, prefix, hint);
 
       name_list_len = name_list.length ();
 
@@ -511,42 +545,42 @@ generate_completion (const std::string& text, int state)
       matches = 0;
 
       for (int i = 0; i < name_list_len; i++)
-	if (hint == name_list[i].substr (0, hint_len))
-	  matches++;
+        if (hint == name_list[i].substr (0, hint_len))
+          matches++;
     }
 
   if (name_list_total_len > 0 && matches > 0)
     {
       while (list_index < name_list_total_len)
-	{
-	  std::string name = name_list[list_index];
+        {
+          std::string name = name_list[list_index];
 
-	  list_index++;
+          list_index++;
 
-	  if (hint == name.substr (0, hint_len))
-	    {
-	      if (list_index <= name_list_len && ! prefix.empty ())
-		retval = prefix + "." + name;
-	      else
-		retval = name;
+          if (hint == name.substr (0, hint_len))
+            {
+              if (list_index <= name_list_len && ! prefix.empty ())
+                retval = prefix + "." + name;
+              else
+                retval = name;
 
-	      // FIXME -- looks_like_struct is broken for now,
-	      // so it always returns false.
+              // FIXME -- looks_like_struct is broken for now,
+              // so it always returns false.
 
-	      if (matches == 1 && looks_like_struct (retval))
- 		{
- 		  // Don't append anything, since we don't know
- 		  // whether it should be '(' or '.'.
+              if (matches == 1 && looks_like_struct (retval))
+                {
+                  // Don't append anything, since we don't know
+                  // whether it should be '(' or '.'.
 
- 		  command_editor::set_completion_append_character ('\0');
- 		}
- 	      else
- 		command_editor::set_completion_append_character
- 		  (Vcompletion_append_char);
+                  command_editor::set_completion_append_character ('\0');
+                }
+              else
+                command_editor::set_completion_append_character
+                  (Vcompletion_append_char);
 
-	      break;
-	    }
-	}
+              break;
+            }
+        }
     }
 
   return retval;
@@ -594,7 +628,7 @@ get_debug_input (const std::string& prompt)
   octave_user_code *caller = octave_call_stack::caller_user_code ();
   std::string nm;
 
-  int curr_debug_line = tree_evaluator::debug_line ();
+  int curr_debug_line = octave_call_stack::current_line ();
 
   bool have_file = false;
 
@@ -603,9 +637,9 @@ get_debug_input (const std::string& prompt)
       nm = caller->fcn_file_name ();
 
       if (nm.empty ())
-	nm = caller->name ();
+        nm = caller->name ();
       else
-	have_file = true;
+        have_file = true;
     }
   else
     curr_debug_line = -1;
@@ -615,93 +649,130 @@ get_debug_input (const std::string& prompt)
   if (! nm.empty ())
     {
       if (Vgud_mode)
-	{
-	  static char ctrl_z = 'Z' & 0x1f;
+        {
+          static char ctrl_z = 'Z' & 0x1f;
 
-	  buf << ctrl_z << ctrl_z << nm << ":" << curr_debug_line;
-	}
+          buf << ctrl_z << ctrl_z << nm << ":" << curr_debug_line;
+        }
       else
-	{
-	  // FIXME -- we should come up with a clean way to detect
-	  // that we are stopped on the no-op command that marks the
-	  // end of a function or script.
+        {
+          // FIXME -- we should come up with a clean way to detect
+          // that we are stopped on the no-op command that marks the
+          // end of a function or script.
 
-	  buf << "stopped in " << nm;
+          buf << "stopped in " << nm;
 
-	  if (curr_debug_line > 0)
-	    buf << " at line " << curr_debug_line;
+          if (curr_debug_line > 0)
+            buf << " at line " << curr_debug_line;
 
-	  if (have_file)
-	    {
-	      std::string line_buf
-		= get_file_line (nm, curr_debug_line);
+          if (have_file)
+            {
+              std::string line_buf
+                = get_file_line (nm, curr_debug_line);
 
-	      if (! line_buf.empty ())
-		buf << "\n" << curr_debug_line << ": " << line_buf;
-	    }
-	}
+              if (! line_buf.empty ())
+                buf << "\n" << curr_debug_line << ": " << line_buf;
+            }
+        }
     }
 
   std::string msg = buf.str ();
 
   if (! msg.empty ())
-    {
-      if (! Vgud_mode)
-	std::cerr << "keyboard: ";
+    std::cerr << msg << std::endl;
 
-      std::cerr << msg << std::endl;
-    }
+  unwind_protect frame;
 
-  unwind_protect::begin_frame ("get_debug_input");
-
-  unwind_protect_str (VPS1);
+  frame.protect_var (VPS1);
   VPS1 = prompt;
 
-  while (Vdebugging)
+  if (stdin_is_tty)
     {
-      reset_error_handler ();
+      if (! (interactive || forced_interactive)
+          || (reading_fcn_file
+              || reading_classdef_file
+              || reading_script_file
+              || get_input_from_eval_string
+              || input_from_startup_file
+              || input_from_command_line_file))
+        {
+          frame.protect_var (forced_interactive);
+          forced_interactive = true;
 
-      reset_parser ();
+          frame.protect_var (reading_fcn_file);
+          reading_fcn_file = false;
 
-      // Save current value of global_command.
-      unwind_protect_ptr (global_command);
+          frame.protect_var (reading_classdef_file);
+          reading_classdef_file = false;
 
-      // Do this with an unwind-protect cleanup function so that the
-      // forced variables will be unmarked in the event of an interrupt.
-      symbol_table::scope_id scope = symbol_table::top_scope ();
-      unwind_protect::add (symbol_table::unmark_forced_variables, &scope);
+          frame.protect_var (reading_script_file);
+          reading_script_file = false;
 
-      // This is the same as yyparse in parse.y.
-      int retval = octave_parse ();
+          frame.protect_var (input_from_startup_file);
+          input_from_startup_file = false;
 
-      if (retval == 0 && global_command)
-	{
-	  global_command->accept (*current_evaluator);
+          frame.protect_var (input_from_command_line_file);
+          input_from_command_line_file = false;
 
-	  // FIXME -- To avoid a memory leak, global_command should be
-	  // deleted, I think.  But doing that here causes trouble if
-	  // an error occurs while executing a debugging command
-	  // (dbstep, for example). It's not clear to me why that
-	  // happens.
-	  //
-	  // delete global_command;
-	  //
-	  // global_command = 0;
+          frame.protect_var (get_input_from_eval_string);
+          get_input_from_eval_string = false;
 
-	  if (octave_completion_matches_called)
-	    octave_completion_matches_called = false;	    
-	}
+          YY_BUFFER_STATE old_buf = current_buffer ();
+          YY_BUFFER_STATE new_buf = create_buffer (get_input_from_stdin ());
 
-      // Unmark forced variables.
-      unwind_protect::run ();
+          // FIXME: are these safe?
+          frame.add_fcn (switch_to_buffer, old_buf);
+          frame.add_fcn (delete_buffer, new_buf);
 
-      // Restore previous value of global_command.
-      unwind_protect::run ();
+          switch_to_buffer (new_buf);
+        }
 
-      OCTAVE_QUIT;
+      while (Vdebugging)
+        {
+          reset_error_handler ();
+
+          reset_parser ();
+
+          // Save current value of global_command.
+          frame.protect_var (global_command);
+
+          global_command = 0;
+
+          // Do this with an unwind-protect cleanup function so that the
+          // forced variables will be unmarked in the event of an interrupt.
+          symbol_table::scope_id scope = symbol_table::top_scope ();
+          frame.add_fcn (symbol_table::unmark_forced_variables, scope);
+
+          // This is the same as yyparse in parse.y.
+          int retval = octave_parse ();
+
+          if (retval == 0 && global_command)
+            {
+              global_command->accept (*current_evaluator);
+
+              // FIXME -- To avoid a memory leak, global_command should be
+              // deleted, I think.  But doing that here causes trouble if
+              // an error occurs while executing a debugging command
+              // (dbstep, for example). It's not clear to me why that
+              // happens.
+              //
+              // delete global_command;
+              //
+              // global_command = 0;
+
+              if (octave_completion_matches_called)
+                octave_completion_matches_called = false;
+            }
+
+          // Unmark forced variables.
+          // Restore previous value of global_command.
+          frame.run_top (2);
+
+          octave_quit ();
+        }
     }
-
-  unwind_protect::run_frame ("get_debug_input");
+  else
+    warning ("invalid attempt to debug script read from stdin");
 }
 
 // If the user simply hits return, this will produce an empty matrix.
@@ -735,35 +806,35 @@ get_user_input (const octave_value_list& args, int nargout)
   if (! (error_state || input_buf.empty ()))
     {
       if (! input_from_startup_file)
-	command_history::add (input_buf);
+        command_history::add (input_buf);
 
       size_t len = input_buf.length ();
 
       octave_diary << input_buf;
 
       if (input_buf[len - 1] != '\n')
-	octave_diary << "\n";
+        octave_diary << "\n";
 
       if (len < 1)
-	return read_as_string ? octave_value ("") : octave_value (Matrix ());
+        return read_as_string ? octave_value ("") : octave_value (Matrix ());
 
       if (read_as_string)
-	{
-	  // FIXME -- fix gnu_readline and octave_gets instead!
-	  if (input_buf.length () == 1 && input_buf[0] == '\n')
-	    retval(0) = "";
-	  else
-	    retval(0) = input_buf;
-	}
+        {
+          // FIXME -- fix gnu_readline and octave_gets instead!
+          if (input_buf.length () == 1 && input_buf[0] == '\n')
+            retval(0) = "";
+          else
+            retval(0) = input_buf;
+        }
       else
-	{
-	  int parse_status = 0;
+        {
+          int parse_status = 0;
 
-	  retval = eval_string (input_buf, true, parse_status, nargout);
+          retval = eval_string (input_buf, true, parse_status, nargout);
 
-	  if (! Vdebugging && retval.length () == 0)
-	    retval(0) = Matrix ();
-	}
+          if (! Vdebugging && retval.length () == 0)
+            retval(0) = Matrix ();
+        }
     }
   else
     error ("input: reading user-input failed!");
@@ -773,7 +844,7 @@ get_user_input (const octave_value_list& args, int nargout)
 
 DEFUN (input, args, nargout,
   "-*- texinfo -*-\n\
-@deftypefn {Built-in Function} {} input (@var{prompt})\n\
+@deftypefn  {Built-in Function} {} input (@var{prompt})\n\
 @deftypefnx {Built-in Function} {} input (@var{prompt}, \"s\")\n\
 Print a prompt and wait for user input.  For example,\n\
 \n\
@@ -829,11 +900,11 @@ octave_yes_or_no (const std::string& prompt)
       std::string input_buf = interactive_input (prompt_string, true);
 
       if (input_buf == "yes")
-	return true;
+        return true;
       else if (input_buf == "no")
-	return false;
+        return false;
       else
-	message (0, "Please answer yes or no.");
+        message (0, "Please answer yes or no.");
     }
 }
 
@@ -856,15 +927,15 @@ RET and can edit it until it has been confirmed.\n\
       std::string prompt;
 
       if (nargin == 1)
-	{
-	  prompt = args(0).string_value ();
+        {
+          prompt = args(0).string_value ();
 
-	  if (error_state)
-	    {
-	      error ("yes_or_no: expecting argument to be character string");
-	      return retval;
-	    }
-	}
+          if (error_state)
+            {
+              error ("yes_or_no: PROMPT must be a character string");
+              return retval;
+            }
+        }
 
       retval = octave_yes_or_no (prompt);
     }
@@ -874,20 +945,6 @@ RET and can edit it until it has been confirmed.\n\
   return retval;
 }
 
-static void
-restore_command_history (void *)
-{
-  command_history::ignore_entries (! Vsaving_history);
-}
-
-static size_t saved_frame = 0;
-
-static void
-restore_frame (void *)
-{
-  octave_call_stack::goto_frame (saved_frame);
-}
-
 octave_value
 do_keyboard (const octave_value_list& args)
 {
@@ -897,21 +954,26 @@ do_keyboard (const octave_value_list& args)
 
   assert (nargin == 0 || nargin == 1);
 
-  unwind_protect::begin_frame ("do_keyboard");
+  unwind_protect frame;
 
   // FIXME -- we shouldn't need both the
   // command_history object and the
   // Vsaving_history variable...
   command_history::ignore_entries (false);
 
-  unwind_protect::add (restore_command_history, 0);
+  frame.add_fcn (command_history::ignore_entries, ! Vsaving_history);
 
-  unwind_protect_bool (Vsaving_history);
-  unwind_protect_bool (Vdebugging);
+  frame.protect_var (Vsaving_history);
+  frame.protect_var (Vdebugging);
 
-  saved_frame = octave_call_stack::current_frame ();
-  unwind_protect::add (restore_frame);
-  unwind_protect_size_t (saved_frame);
+  frame.add_fcn (octave_call_stack::restore_frame,
+                 octave_call_stack::current_frame ());
+
+  // FIXME -- probably we just want to print one line, not the
+  // entire statement, which might span many lines...
+  //
+  // tree_print_code tpc (octave_stdout);
+  // stmt.accept (tpc);
 
   Vsaving_history = true;
   Vdebugging = true;
@@ -923,8 +985,6 @@ do_keyboard (const octave_value_list& args)
   if (! error_state)
     get_debug_input (prompt);
 
-  unwind_protect::run_frame ("do_keyboard");
-
   return retval;
 }
 
@@ -951,16 +1011,19 @@ If @code{keyboard} is invoked without arguments, a default prompt of\n\
 
   if (nargin == 0 || nargin == 1)
     {
-      saved_frame = octave_call_stack::current_frame ();
-      unwind_protect::add (restore_frame);
-      unwind_protect_size_t (saved_frame);
+      unwind_protect frame;
+
+      frame.add_fcn (octave_call_stack::restore_frame,
+                     octave_call_stack::current_frame ());
 
       // Skip the frame assigned to the keyboard function.
-      octave_call_stack::goto_frame_relative (0, true);
+      octave_call_stack::goto_frame_relative (0);
+
+      tree_evaluator::debug_mode = true;
+
+      tree_evaluator::current_frame = octave_call_stack::current_frame ();
 
       do_keyboard (args);
-
-      unwind_protect::run ();
     }
   else
     print_usage ();
@@ -970,7 +1033,7 @@ If @code{keyboard} is invoked without arguments, a default prompt of\n\
 
 DEFUN (echo, args, ,
   "-*- texinfo -*-\n\
-@deffn {Command} echo options\n\
+@deftypefn {Command} {} echo options\n\
 Control whether commands are displayed as they are executed.  Valid\n\
 options are:\n\
 \n\
@@ -992,7 +1055,7 @@ functions.\n\
 \n\
 @noindent\n\
 With no arguments, @code{echo} toggles the current echo state.\n\
-@end deffn")
+@end deftypefn")
 {
   octave_value_list retval;
 
@@ -1007,40 +1070,40 @@ With no arguments, @code{echo} toggles the current echo state.\n\
     {
     case 1:
       {
-	if ((Vecho_executing_commands & ECHO_SCRIPTS)
-	    || (Vecho_executing_commands & ECHO_FUNCTIONS))
-	  Vecho_executing_commands = ECHO_OFF;
-	else
-	  Vecho_executing_commands = ECHO_SCRIPTS;
+        if ((Vecho_executing_commands & ECHO_SCRIPTS)
+            || (Vecho_executing_commands & ECHO_FUNCTIONS))
+          Vecho_executing_commands = ECHO_OFF;
+        else
+          Vecho_executing_commands = ECHO_SCRIPTS;
       }
       break;
 
     case 2:
       {
-	std::string arg = argv[1];
+        std::string arg = argv[1];
 
-	if (arg == "on")
-	  Vecho_executing_commands = ECHO_SCRIPTS;
-	else if (arg == "off")
-	  Vecho_executing_commands = ECHO_OFF;
-	else
-	  print_usage ();
+        if (arg == "on")
+          Vecho_executing_commands = ECHO_SCRIPTS;
+        else if (arg == "off")
+          Vecho_executing_commands = ECHO_OFF;
+        else
+          print_usage ();
       }
       break;
 
     case 3:
       {
-	std::string arg = argv[1];
+        std::string arg = argv[1];
 
-	if (arg == "on" && argv[2] == "all")
-	  {
-	    int tmp = (ECHO_SCRIPTS | ECHO_FUNCTIONS);
-	    Vecho_executing_commands = tmp;
-	  }
-	else if (arg == "off" && argv[2] == "all")
-	  Vecho_executing_commands = ECHO_OFF;
-	else
-	  print_usage ();
+        if (arg == "on" && argv[2] == "all")
+          {
+            int tmp = (ECHO_SCRIPTS | ECHO_FUNCTIONS);
+            Vecho_executing_commands = tmp;
+          }
+        else if (arg == "off" && argv[2] == "all")
+          Vecho_executing_commands = ECHO_OFF;
+        else
+          print_usage ();
       }
       break;
 
@@ -1072,55 +1135,55 @@ a feature, not a bug.\n\
       std::string hint = args(0).string_value ();
 
       if (! error_state)
-	{
-	  int n = 32;
+        {
+          int n = 32;
 
-	  string_vector list (n);
+          string_vector list (n);
 
-	  int k = 0;
+          int k = 0;
 
-	  for (;;)
-	    {
-	      std::string cmd = generate_completion (hint, k);
+          for (;;)
+            {
+              std::string cmd = generate_completion (hint, k);
 
-	      if (! cmd.empty ())
-		{
-		  if (k == n)
-		    {
-		      n *= 2;
-		      list.resize (n);
-		    }
+              if (! cmd.empty ())
+                {
+                  if (k == n)
+                    {
+                      n *= 2;
+                      list.resize (n);
+                    }
 
-		  list[k++] = cmd;
-		}
-	      else
-		{
-		  list.resize (k);
-		  break;
-		}
-	    }
+                  list[k++] = cmd;
+                }
+              else
+                {
+                  list.resize (k);
+                  break;
+                }
+            }
 
-	  if (nargout > 0)
-	    {
-	      if (! list.empty ())
-		retval = list;
-	      else
-		retval = "";
-	    }
-	  else
-	    {
-	      // We don't use string_vector::list_in_columns here
-	      // because it will be easier for Emacs if the names
-	      // appear in a single column.
+          if (nargout > 0)
+            {
+              if (! list.empty ())
+                retval = list;
+              else
+                retval = "";
+            }
+          else
+            {
+              // We don't use string_vector::list_in_columns here
+              // because it will be easier for Emacs if the names
+              // appear in a single column.
 
-	      int len = list.length ();
+              int len = list.length ();
 
-	      for (int i = 0; i < len; i++)
-		octave_stdout << list[i] << "\n";
-	    }
+              for (int i = 0; i < len; i++)
+                octave_stdout << list[i] << "\n";
+            }
 
-	  octave_completion_matches_called = true;
-	}
+          octave_completion_matches_called = true;
+        }
     }
   else
     print_usage ();
@@ -1150,7 +1213,7 @@ for details.\n\
       std::string file = args(0).string_value ();
 
       if (! error_state)
-	command_editor::read_init_file (file);
+        command_editor::read_init_file (file);
     }
   else
     print_usage ();
@@ -1183,38 +1246,43 @@ static hook_fcn_map_type hook_fcn_map;
 static int
 input_event_hook (void)
 {
-  hook_fcn_map_type::iterator p = hook_fcn_map.begin ();
-
-  while (p != hook_fcn_map.end ())
+  if (! lexer_flags.defining_func)
     {
-      std::string hook_fcn = p->first;
-      octave_value user_data = p->second;
+      hook_fcn_map_type::iterator p = hook_fcn_map.begin ();
 
-      p++;
+      while (p != hook_fcn_map.end ())
+        {
+          std::string hook_fcn = p->first;
+          octave_value user_data = p->second;
 
-      if (is_valid_function (hook_fcn))
-	{
-	  if (user_data.is_defined ())
-	    feval (hook_fcn, user_data, 0);
-	  else
-	    feval (hook_fcn, octave_value_list (), 0);
-	}
-      else
-	hook_fcn_map.erase (p);
+          hook_fcn_map_type::iterator q = p++;
+
+          if (is_valid_function (hook_fcn))
+            {
+              if (user_data.is_defined ())
+                feval (hook_fcn, user_data, 0);
+              else
+                feval (hook_fcn, octave_value_list (), 0);
+            }
+          else
+            hook_fcn_map.erase (q);
+        }
+
+      if (hook_fcn_map.empty ())
+        command_editor::remove_event_hook (input_event_hook);
     }
 
-  if (hook_fcn_map.empty ())
-    command_editor::remove_event_hook (input_event_hook);
-
   return 0;
 }
 
 DEFUN (add_input_event_hook, args, ,
   "-*- texinfo -*-\n\
-@deftypefn {Built-in Function} {} add_input_event_hook (@var{fcn}, @var{data})\n\
+@deftypefn  {Built-in Function} {} add_input_event_hook (@var{fcn})\n\
+@deftypefnx {Built-in Function} {} add_input_event_hook (@var{fcn}, @var{data})\n\
 Add the named function @var{fcn} to the list of functions to call\n\
 periodically when Octave is waiting for input.  The function should\n\
 have the form\n\
+\n\
 @example\n\
 @var{fcn} (@var{data})\n\
 @end example\n\
@@ -1233,19 +1301,19 @@ arguments.\n\
       octave_value user_data;
 
       if (nargin == 2)
-	user_data = args(1);
+        user_data = args(1);
 
       std::string hook_fcn = args(0).string_value ();
 
       if (! error_state)
-	{
-	  if (hook_fcn_map.empty ())
-	    command_editor::add_event_hook (input_event_hook);
+        {
+          if (hook_fcn_map.empty ())
+            command_editor::add_event_hook (input_event_hook);
 
-	  hook_fcn_map[hook_fcn] = user_data;
-	}
+          hook_fcn_map[hook_fcn] = user_data;
+        }
       else
-	error ("add_input_event_hook: expecting string as first arg");
+        error ("add_input_event_hook: expecting string as first arg");
     }
   else
     print_usage ();
@@ -1256,7 +1324,7 @@ arguments.\n\
 DEFUN (remove_input_event_hook, args, ,
   "-*- texinfo -*-\n\
 @deftypefn {Built-in Function} {} remove_input_event_hook (@var{fcn})\n\
-Remove the named function @var{fcn} to the list of functions to call\n\
+Remove the named function @var{fcn} from the list of functions to call\n\
 periodically when Octave is waiting for input.\n\
 @seealso{add_input_event_hook}\n\
 @end deftypefn")
@@ -1270,20 +1338,20 @@ periodically when Octave is waiting for input.\n\
       std::string hook_fcn = args(0).string_value ();
 
       if (! error_state)
-	{
-	  hook_fcn_map_type::iterator p = hook_fcn_map.find (hook_fcn);
+        {
+          hook_fcn_map_type::iterator p = hook_fcn_map.find (hook_fcn);
 
-	  if (p != hook_fcn_map.end ())
-	    hook_fcn_map.erase (p);
-	  else
-	    error ("remove_input_event_hook: %s not found in list",
-		   hook_fcn.c_str ());
+          if (p != hook_fcn_map.end ())
+            hook_fcn_map.erase (p);
+          else
+            error ("remove_input_event_hook: %s not found in list",
+                   hook_fcn.c_str ());
 
-	  if (hook_fcn_map.empty ())
-	    command_editor::remove_event_hook (input_event_hook);
-	}
+          if (hook_fcn_map.empty ())
+            command_editor::remove_event_hook (input_event_hook);
+        }
       else
-	error ("remove_input_event_hook: expecting string as first arg");
+        error ("remove_input_event_hook: expecting string as first arg");
     }
   else
     print_usage ();
@@ -1293,7 +1361,7 @@ periodically when Octave is waiting for input.\n\
 
 DEFUN (PS1, args, nargout,
   "-*- texinfo -*-\n\
-@deftypefn {Built-in Function} {@var{val} =} PS1 ()\n\
+@deftypefn  {Built-in Function} {@var{val} =} PS1 ()\n\
 @deftypefnx {Built-in Function} {@var{old_val} =} PS1 (@var{new_val})\n\
 Query or set the primary prompt string.  When executing interactively,\n\
 Octave displays the primary prompt when it is ready to read a command.\n\
@@ -1302,15 +1370,24 @@ The default value of the primary prompt string is @code{\"\\s:\\#> \"}.\n\
 To change it, use a command like\n\
 \n\
 @example\n\
-octave:13> PS1 (\"\\\\u@@\\\\H> \")\n\
+PS1 (\"\\\\u@@\\\\H> \")\n\
 @end example\n\
 \n\
 @noindent\n\
 which will result in the prompt @samp{boris@@kremvax> } for the user\n\
 @samp{boris} logged in on the host @samp{kremvax.kgb.su}.  Note that two\n\
 backslashes are required to enter a backslash into a double-quoted\n\
-character string.\n\
-@xref{Strings}.\n\
+character string.  @xref{Strings}.\n\
+\n\
+You can also use ANSI escape sequences if your terminal supports them.\n\
+This can be useful for coloring the prompt.  For example,\n\
+\n\
+@example\n\
+PS1 (\"\\\\[\\\\033[01;31m\\\\]\\\\s:\\\\#> \\\\[\\\\033[0m\\]\")\n\
+@end example\n\
+\n\
+@noindent\n\
+will give the default Octave prompt a red coloring.\n\
 @seealso{PS2, PS4}\n\
 @end deftypefn")
 {
@@ -1319,7 +1396,7 @@ character string.\n\
 
 DEFUN (PS2, args, nargout,
   "-*- texinfo -*-\n\
-@deftypefn {Built-in Function} {@var{val} =} PS2 ()\n\
+@deftypefn  {Built-in Function} {@var{val} =} PS2 ()\n\
 @deftypefnx {Built-in Function} {@var{old_val} =} PS2 (@var{new_val})\n\
 Query or set the secondary prompt string.  The secondary prompt is\n\
 printed when Octave is expecting additional input to complete a\n\
@@ -1335,7 +1412,7 @@ string is @code{\"> \"}.\n\
 
 DEFUN (PS4, args, nargout,
   "-*- texinfo -*-\n\
-@deftypefn {Built-in Function} {@var{val} =} PS4 ()\n\
+@deftypefn  {Built-in Function} {@var{val} =} PS4 ()\n\
 @deftypefnx {Built-in Function} {@var{old_val} =} PS4 (@var{new_val})\n\
 Query or set the character string used to prefix output produced\n\
 when echoing commands is enabled.\n\
@@ -1349,7 +1426,7 @@ The default value is @code{\"+ \"}.\n\
 
 DEFUN (completion_append_char, args, nargout,
   "-*- texinfo -*-\n\
-@deftypefn {Built-in Function} {@var{val} =} completion_append_char ()\n\
+@deftypefn  {Built-in Function} {@var{val} =} completion_append_char ()\n\
 @deftypefnx {Built-in Function} {@var{old_val} =} completion_append_char (@var{new_val})\n\
 Query or set the internal character variable that is appended to\n\
 successful command-line completion attempts.  The default\n\
@@ -1361,7 +1438,7 @@ value is @code{\" \"} (a single space).\n\
 
 DEFUN (echo_executing_commands, args, nargout,
   "-*- texinfo -*-\n\
-@deftypefn {Built-in Function} {@var{val} =} echo_executing_commands ()\n\
+@deftypefn  {Built-in Function} {@var{val} =} echo_executing_commands ()\n\
 @deftypefnx {Built-in Function} {@var{old_val} =} echo_executing_commands (@var{new_val})\n\
 Query or set the internal variable that controls the echo state.\n\
 It may be the sum of the following values:\n\
@@ -1381,7 +1458,7 @@ More than one state can be active at once.  For example, a value of 3 is\n\
 equivalent to the command @kbd{echo on all}.\n\
 \n\
 The value of @code{echo_executing_commands} may be set by the @kbd{echo}\n\
-command or the command line option @code{--echo-commands}.\n\
+command or the command line option @option{--echo-commands}.\n\
 @end deftypefn")
 {
   return SET_INTERNAL_VARIABLE (echo_executing_commands);
@@ -1389,7 +1466,7 @@ command or the command line option @code{--echo-commands}.\n\
 
 DEFUN (__request_drawnow__, args, ,
   "-*- texinfo -*-\n\
-@deftypefn {Built-in Function} {} __request_drawnow__ ()\n\
+@deftypefn  {Built-in Function} {} __request_drawnow__ ()\n\
 @deftypefnx {Built-in Function} {} __request_drawnow__ (@var{flag})\n\
 Undocumented internal function.\n\
 @end deftypefn")
@@ -1430,10 +1507,11 @@ Undocumented internal function.\n\
 
 DEFUN (filemarker, args, nargout,
   "-*- texinfo -*-\n\
-@deftypefn {Built-in Function} {} filemarker ()\n\
-Returns or sets the character used to separate filename from the\n\
+@deftypefn  {Built-in Function} {@var{val} =} filemarker ()\n\
+@deftypefnx {Built-in Function} {} filemarker (@var{new_val})\n\
+Query or set the character used to separate filename from the\n\
 the subfunction names contained within the file.  This can be used in\n\
-a generic manner to interact with subfunctions.  For example\n\
+a generic manner to interact with subfunctions.  For example,\n\
 \n\
 @example\n\
 help ([\"myfunc\", filemarker, \"mysubfunc\"])\n\
@@ -1443,7 +1521,7 @@ help ([\"myfunc\", filemarker, \"mysubfunc\"])\n\
 returns the help string associated with the sub-function @code{mysubfunc}\n\
 of the function @code{myfunc}.  Another use of @code{filemarker} is when\n\
 debugging it allows easier placement of breakpoints within sub-functions.\n\
-For example\n\
+For example,\n\
 \n\
 @example\n\
 dbstop ([\"myfunc\", filemarker, \"mysubfunc\"])\n\
@@ -1465,9 +1543,3 @@ will set a breakpoint at the first line of the subfunction @code{mysubfunc}.\n\
 
   return retval;
 }
-
-/*
-;;; Local Variables: ***
-;;; mode: C++ ***
-;;; End: ***
-*/
