@@ -121,12 +121,15 @@ octave_class::get_current_method_class (void)
 {
   std::string retval = class_name ();
 
-  octave_function *fcn = octave_call_stack::current ();
+  if (nparents () > 0)
+    {
+      octave_function *fcn = octave_call_stack::current ();
 
-  // Here we are just looking to see if FCN is a method or constructor
-  // for any class, not specifically this one.
-  if (fcn->is_class_method () || fcn->is_class_constructor ())
-    retval = fcn->dispatch_class ();
+      // Here we are just looking to see if FCN is a method or constructor
+      // for any class, not specifically this one.
+      if (fcn && (fcn->is_class_method () || fcn->is_class_constructor ()))
+        retval = fcn->dispatch_class ();
+    }
 
   return retval;
 }
@@ -1697,7 +1700,7 @@ octave_class::exemplar_info::exemplar_info (const octave_value& obj)
       parent_class_names = obj.parent_class_name_list ();
     }
   else
-    error ("invalid call to exmplar_info constructor");
+    error ("invalid call to exemplar_info constructor");
 }
 
 
@@ -1826,7 +1829,7 @@ derived.\n\
                         }
                     }
                   else
-                    error ("class: expecting structure as first argument");
+                    error ("class: expecting structure S as first argument");
                 }
               else
                 error ("class: `%s' is invalid as a class name in this context",
@@ -1836,7 +1839,7 @@ derived.\n\
             error ("class: invalid call from outside class constructor or method");
         }
       else
-        error ("class: expecting character string as second argument");
+        error ("class: ID (class name) must be a character string");
     }
 
   return retval;
