@@ -1,7 +1,6 @@
 /*
 
-Copyright (C) 1994, 1995, 1996, 1997, 1999, 2002, 2003, 2004, 2005,
-              2006, 2007 John W. Eaton
+Copyright (C) 1994-2011 John W. Eaton
 
 This file is part of Octave.
 
@@ -37,6 +36,7 @@ class tree_walker;
 
 #include "base-list.h"
 #include "pt-decl.h"
+#include "symtab.h"
 
 // Parameter lists.  Used to hold the list of input and output
 // parameters in a function definition.  Elements are identifiers
@@ -47,6 +47,12 @@ tree_parameter_list : public octave_base_list<tree_decl_elt *>
 {
 public:
 
+  enum in_or_out
+    {
+      in = 1,
+      out = 2
+    };
+
   tree_parameter_list (void)
     : marked_for_varargs (0) { }
 
@@ -57,16 +63,14 @@ public:
 
   void mark_as_formal_parameters (void);
 
-  void mark_varargs (void) { marked_for_varargs = 1; }
+  bool validate (in_or_out type);
 
   bool takes_varargs (void) const { return marked_for_varargs != 0; }
 
-  void mark_varargs_only (void) { marked_for_varargs = -1; }
-
   bool varargs_only (void) { return (marked_for_varargs < 0); }
 
   void initialize_undefined_elements (const std::string& warnfor,
-				      int nargout, const octave_value& val);
+                                      int nargout, const octave_value& val);
 
   void define_from_arg_vector (const octave_value_list& args);
 
@@ -74,9 +78,10 @@ public:
 
   bool is_defined (void);
 
-  octave_value_list convert_to_const_vector (const Cell& varargout);
+  octave_value_list convert_to_const_vector (int nargout, const Cell& varargout);
 
-  tree_parameter_list *dup (symbol_table *sym_tab);
+  tree_parameter_list *dup (symbol_table::scope_id scope,
+                            symbol_table::context_id context) const;
 
   void accept (tree_walker& tw);
 
@@ -84,6 +89,10 @@ private:
 
   int marked_for_varargs;
 
+  void mark_varargs (void) { marked_for_varargs = 1; }
+
+  void mark_varargs_only (void) { marked_for_varargs = -1; }
+
   // No copying!
 
   tree_parameter_list (const tree_parameter_list&);
@@ -105,7 +114,8 @@ public:
 
   ~tree_return_list (void);
 
-  tree_return_list *dup (symbol_table *sym_tab);
+  tree_return_list *dup (symbol_table::scope_id scope,
+                         symbol_table::context_id context) const;
 
   void accept (tree_walker& tw);
 
@@ -137,9 +147,3 @@ private:
 };
 
 #endif
-
-/*
-;;; Local Variables: ***
-;;; mode: C++ ***
-;;; End: ***
-*/
