@@ -1,7 +1,6 @@
 /*
 
-Copyright (C) 1996, 1997, 2000, 2001, 2002, 2004, 2005, 2006, 2007
-              John W. Eaton
+Copyright (C) 1996-2011 John W. Eaton
 
 This file is part of Octave.
 
@@ -33,18 +32,6 @@ along with Octave; see the file COPYING.  If not, see
 #include "pt.h"
 #include "pt-pr-code.h"
 
-// If true, stop executing at the next possible point.
-bool tree::break_next = false;
-
-// The line where dbnext was executed.
-int tree::last_line = 0;
-
-// The function where the last breakpoint occurred.
-const octave_function *tree::break_function = 0;
-
-// The statement where the last breakpoint occurred.
-const tree *tree::break_statement = 0;
-
 // Hide the details of the string buffer so that we are less likely to
 // create a memory leak.
 
@@ -61,9 +48,3 @@ tree::str_print_code (void)
 
   return retval;
 }
-
-/*
-;;; Local Variables: ***
-;;; mode: C++ ***
-;;; End: ***
-*/
